// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"access-control-monitor/consts"
	"access-control-monitor/errors"
	"access-control-monitor/impl/v1"
	"access-control-monitor/model/v1"
	"access-control-monitor/param/base"
	param_v1 "access-control-monitor/param/v1"
	"access-control-monitor/utils"
	"bytes"
	"encoding/base64"
	"fmt"
	"net/http"

	"google.golang.org/grpc/status"

	"github.com/jaryhe/gopkgs/logger"
	"github.com/jaryhe/gopkgs/tasker/httptasker"
	"github.com/jaryhe/gopkgs/util"

	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)

// 授权获取key
// @Summary 授权获取key
// @Description 授权获取key
// @Tags user
// @Accept  json
// @Produce  json
// @Param body body v1.LoginBody true "授权信息"
// @Success 200 {object} v1.LoginResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/auth [put]
func (c *Controller) Auth(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.LoginRequest{}
	resp := param_v1.LoginResponse{Result: base.Result{
		Code:    0,
		Message: "成功",
	}}

	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.LoginBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		// 第一步本地缓存获取，本地缓存有直接返回，本地缓存无调用接口获取
		data, err := utils.GetDeviceInfo(req.Sn)
		if err != nil {
			return err
		}

		resp.Data.Key = data.Key
		ctx.JSON(http.StatusOK, resp)

		return nil
	}
	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// 上传员工信息
// @Summary 上传员工信息
// @Description 上传员工信息
// @Tags user
// @Accept  json
// @Produce  json
// @Param body body v1.RegisterRequest true "上传员工信息"
// @Success 200 {object} v1.RegisterResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/staff [post]
func (c *Controller) Register(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.RegisterRequest{}
	resp := param_v1.RegisterResponse{Result: base.Result{
		Code:    0,
		Message: "成功",
	}}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.RegisterBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		bytes, _ := json.Marshal(req)
		fmt.Printf("register read:%s\n", bytes)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		// 第一步校验key
		value, err := utils.GetDeviceInfo(req.Sn)
		if err != nil {
			return err
		}

		if req.Sn == "" {
			return status.Error(10003, "参数错误,设备sn码不能为空")
		}
		if req.WorkNo == "" {
			return status.Error(10003, "参数错误,工号不能为空")
		}

		if req.CorpName == "" {
			return status.Error(10003, "参数错误,班组所在企业名称不能为空")
		}
		if req.CorpCode == "" {
			return status.Error(10003, "参数错误,班组所在企业统一社会信用代码不能为空")
		}
		if req.TeamSysNo == 0 {
			return status.Error(10003, "参数错误,班组号不能为空")
		}
		if req.TeamName == "" {
			return status.Error(10003, "参数错误,班组名称不能为空")
		}
		if req.WorkName == "" {
			return status.Error(10003, "参数错误,工人姓名不能为空")
		}
		if _, ok := consts.IDCardType[req.IDCardType]; !ok {
			return status.Error(10003, "参数错误,错误的证件类型")
		}
		if req.IDCardNumber == "" {
			return status.Error(10003, "参数错误,证件号码不能为空")
		}
		if req.Age < 1 {
			return status.Error(10003, "年龄不能为空")
		}
		if req.Gender != "男" && req.Gender != "女" {
			return status.Error(10003, "性别只能为男或女")
		}
		if _, ok := consts.NationIntMap[req.Nation]; !ok {
			return status.Error(10003, "民族错误")
		}
		if req.Address == "" {
			return status.Error(10003, "住址不能为空")
		}
		if req.HeadImage == "" {
			return status.Error(10003, "头像不能为空")
		}
		if _, ok := consts.PoliticsType[req.PoliticsType]; !ok {
			return status.Error(10003, "政治面貌错误")
		}
		if _, ok := consts.CultureLevelType[req.CultureLevelType]; !ok {
			return status.Error(10003, "文化程度错误")
		}
		if req.GrantOrg == "" {
			return status.Error(10003, "发证机关不能为空")
		}
		if req.NativePlace == "" {
			return status.Error(10003, "籍贯不能为空")
		}
		if req.Mobile == "" {
			return status.Error(10003, "手机号不能为空")
		}
		if req.IsTeamLeader != "是" && req.IsTeamLeader != "否" {
			return status.Error(10003, "is_team_leader只能为是或否")
		}
		if req.HasContract != "是" && req.HasContract != "否" {
			return status.Error(10003, "has_contract只能为是或否")
		}
		if !utils.VerifyMobileFormat(req.Mobile) {
			return status.Error(10003, "电话号码格式错误")
		}
		workTypeMap, err := model.GetWorkItemMap()
		if err != nil {
			logger.Error("func",
				zap.String("call", "model.GetWorkItemMap"),
				zap.String("error", err.Error()))
			return errors.DataBaseError
		}
		if _, ok := workTypeMap[req.WorkType]; !ok && req.TeamSysNo != -100{
			return status.Error(10003, "工种错误")
		}


		// 上传身份证照片
		photo, err := base64.StdEncoding.DecodeString(req.HeadImage)
		if err != nil {
			logger.Error("func",
				zap.String("call", "base64.StdEncoding.DecodeString"),
				zap.String("args", "headImage"+req.WorkName),
				zap.String("error", err.Error()))
			return status.Error(10003, "参数错误,图片base64编码格式不对")
		}
		reader := bytes.NewReader(photo)
		req.HeadImage, err = utils.UploadToMinio(req.WorkName+"头像", reader, reader.Size(), "image/png")
		if err != nil {
			logger.Error("func",
				zap.String("call", "UploadToMinio"),
				zap.String("args", req.WorkName+"头像"),
				zap.String("error", err.Error()))
			return errors.SystemError
		}

		// 注册
		err = v1.StaffRegister(req.RegisterBody, value.ProjectId, value.ProjectCode)
		if err != nil {
			return err
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}
	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}


//
// @Summary 更新员工信息
// @Description 更新员工信息
// @Tags user
// @Accept  json
// @Produce  json
// @Param body body v1.RegisterBody true "上传员工信息"
// @Success 200 {object} v1.StaffUpdateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/staff [put]
func (c *Controller) StaffUpdate(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.StaffUpdateRequest{}
	resp := param_v1.StaffUpdateResponse{Result: base.Result{
		Code:    0,
		Message: "成功",
	}}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.RegisterBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		bytes, _ := json.Marshal(req)
		fmt.Printf("register read:%s\n", bytes)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		// 第一步校验key
		value, err := utils.GetDeviceInfo(req.Sn)
		if err != nil {
			return err
		}

		if req.Sn == "" {
			return status.Error(10003, "参数错误,设备sn码不能为空")
		}
		if req.WorkNo == "" {
			return status.Error(10003, "参数错误,工号不能为空")
		}

		if req.CorpName == "" {
			return status.Error(10003, "参数错误,班组所在企业名称不能为空")
		}
		if req.CorpCode == "" {
			return status.Error(10003, "参数错误,班组所在企业统一社会信用代码不能为空")
		}
		if req.TeamSysNo == 0 {
			return status.Error(10003, "参数错误,班组号不能为空")
		}
		if req.TeamName == "" {
			return status.Error(10003, "参数错误,班组名称不能为空")
		}
		if req.WorkName == "" {
			return status.Error(10003, "参数错误,工人姓名不能为空")
		}
		if _, ok := consts.IDCardType[req.IDCardType]; !ok {
			return status.Error(10003, "参数错误,错误的证件类型")
		}
		if req.IDCardNumber == "" {
			return status.Error(10003, "参数错误,证件号码不能为空")
		}
		if req.Age < 1 {
			return status.Error(10003, "年龄不能为空")
		}
		if req.Gender != "男" && req.Gender != "女" {
			return status.Error(10003, "性别只能为男或女")
		}
		if _, ok := consts.NationIntMap[req.Nation]; !ok {
			return status.Error(10003, "民族错误")
		}
		if req.Address == "" {
			return status.Error(10003, "住址不能为空")
		}
		if req.HeadImage == "" {
			return status.Error(10003, "头像不能为空")
		}
		if _, ok := consts.PoliticsType[req.PoliticsType]; !ok {
			return status.Error(10003, "政治面貌错误")
		}
		if _, ok := consts.CultureLevelType[req.CultureLevelType]; !ok {
			return status.Error(10003, "文化程度错误")
		}
		if req.GrantOrg == "" {
			return status.Error(10003, "发证机关不能为空")
		}
		if req.NativePlace == "" {
			return status.Error(10003, "籍贯不能为空")
		}
		if req.Mobile == "" {
			return status.Error(10003, "手机号不能为空")
		}
		if req.IsTeamLeader != "是" && req.IsTeamLeader != "否" {
			return status.Error(10003, "is_team_leader只能为是或否")
		}
		if req.HasContract != "是" && req.HasContract != "否" {
			return status.Error(10003, "has_contract只能为是或否")
		}
		workTypeMap, err := model.GetWorkItemMap()
		if err != nil {
			logger.Error("func",
				zap.String("call", "model.GetWorkItemMap"),
				zap.String("error", err.Error()))
			return errors.DataBaseError
		}
		if _, ok := workTypeMap[req.WorkType]; !ok && req.TeamSysNo != -100{
			return status.Error(10003, "工种错误")
		}


		// 上传身份证照片
		photo, err := base64.StdEncoding.DecodeString(req.HeadImage)
		if err != nil {
			logger.Error("func",
				zap.String("call", "base64.StdEncoding.DecodeString"),
				zap.String("args", "headImage"+req.WorkName),
				zap.String("error", err.Error()))
			return status.Error(10003, "参数错误,图片base64编码格式不对")
		}
		reader := bytes.NewReader(photo)
		req.HeadImage, err = utils.UploadToMinio(req.WorkName+"头像", reader, reader.Size(), "image/png")
		if err != nil {
			logger.Error("func",
				zap.String("call", "UploadToMinio"),
				zap.String("args", req.WorkName+"头像"),
				zap.String("error", err.Error()))
			return errors.SystemError
		}

		// 注册
		err = v1.StaffUpdate(req.RegisterBody, value.ProjectId, value.ProjectCode)
		if err != nil {
			return err
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}
	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// 删除员工信息
// @Summary 删除员工信息
// @Description 删除员工信息
// @Tags user
// @Accept  json
// @Produce  json
// @Param body body v1.StaffDeleteBody true "删除人员"
// @Success 200 {object} v1.StaffDeleteResp
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/staff/del [put]
func (c *Controller) StaffDelete(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.StaffDeleteRequest{}
	resp := param_v1.StaffDeleteResponse{Result: base.Result{
		Code:    0,
		Message: "成功",
	}}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.StaffDeleteBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		bytes, _ := json.Marshal(req)
		fmt.Printf("delete read:%s\n", bytes)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 第一步校验key
		info, err := utils.GetDeviceInfo(req.Sn)
		if err != nil {
			return err
		}

		// 响应数据
		err = v1.StaffDelete(req.WorkNos, info.ProjectId, info.ProjectCode)
		if err != nil {
			return err
		}
		//resp.Data.WorkNos = workNos
		ctx.JSON(http.StatusOK, resp)
		return nil
	}
	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// 上传考勤数据
// @Summary 上传考勤数据
// @Description 上传考勤数据
// @Tags user
// @Accept  json
// @Produce  json
// @Param body body v1.AttendanceRequest true "上传考勤数据"
// @Success 200 {object} v1.AttendanceResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/attendance [put]
func (c *Controller) Attendance(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.AttendanceRequest{}
	resp := param_v1.AttendanceResponse{Result: base.Result{
		Code:    0,
		Message: "成功",
	}}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.AttendanceBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		bytes, _ := json.Marshal(req)
		fmt.Printf("attendance read:%s\n", bytes)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		if req.WorkNo == "" {
			return status.Error(10003, "参数错误,工号不能为空")
		}
		if req.TeamSysNo == 0 {
			return status.Error(10003, "参数错误,班组号不能为空")
		}
		if req.TeamName == "" {
			return status.Error(10003, "参数错误,班组名称不能为空")
		}
		if req.WorkName == "" {
			return status.Error(10003, "参数错误,工人姓名不能为空")
		}
		if _, ok := consts.IDCardType[req.IDCardType]; !ok {
			return status.Error(10003, "参数错误,错误的证件类型")
		}
		if req.IDCardNumber == "" {
			return status.Error(10003, "参数错误,证件号码不能为空")
		}
		if req.Datetime == "" {
			return status.Error(10003, "考勤时间不能为空")
		}
		if req.Direction != 1 && req.Direction != 2 {
			return status.Error(10003, "进出方向编码错误")
		}

		if req.Sn == "" {
			return status.Error(10003, "参数错误 设备sn码不能为空")
		}

		// 响应数据
		// 第一步校验key
		value, err := utils.GetDeviceInfo(req.Sn)
		if err != nil {
			return err
		}

		// 添加考勤数据
		err = v1.AddAttendance(req.AttendanceBody, value.ProjectId, value.ProjectCode)
		if err != nil {
			return err
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}
	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

