// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"access-control-monitor/errors"
	"access-control-monitor/impl/v1"
	"access-control-monitor/param/base"
	param_v1 "access-control-monitor/param/v1"
	"access-control-monitor/utils"
	"fmt"
	"net/http"

	"google.golang.org/grpc/status"

	"github.com/jaryhe/gopkgs/logger"
	"github.com/jaryhe/gopkgs/tasker/httptasker"
	"github.com/jaryhe/gopkgs/util"

	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)


//
// @Summary 上传班组信息
// @Description 上传班组信息
// @Tags user
// @Accept  json
// @Produce  json
// @Param body body v1.TeamInfoBody true "上传班组信息"
// @Success 200 {object} v1.TeamInfoResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/team [post]
func (c *Controller) TeamInfoAdd(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.TeamInfoRequest{}
	resp := param_v1.TeamInfoResponse{Result: base.Result{
		Code:    0,
		Message: "成功",
	}}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.TeamInfoBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		bytes, _ := json.Marshal(req)
		fmt.Printf("register read:%s\n", bytes)
		if req.TeamSysNo == 0 {
			return status.Error(10003, "参数错误 班组号不能为空")
		}
		if req.TeamSysNo == -100 {
			req.TeamName = "管理班组"
		}
		if req.TeamName == "" {
			return status.Error(10003, "参数错误 班组名称不能为空")
		}
		if req.TeamLeaderPhone == ""{
			return status.Error(10003, "参数错误 班组长联系电话不能为空")
		}
		if req.TeamLeaderName == "" {
			return status.Error(10003, "参数错误 班组长名称不能为空")
		}
		if req.CorpCode == "" && req.TeamSysNo != 100 {
			 return status.Error(10003, "参数错误 班组所属企业社会信用码不能为空")
		}
		if req.CorpName == "" && req.TeamSysNo != 100 {
			return status.Error(10003, "参数错误 班组所属企业不能为空")
		}
		if req.Sn == "" {
			return status.Error(10003, "参数错误 设备sn码不能为空")
		}
		if !utils.VerifyMobileFormat(req.TeamLeaderPhone) {
			return status.Error(10003, "电话号码格式错误")
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// todo 设备类型
		value, err := utils.GetDeviceInfo(req.Sn)
		if err != nil {
			return err
		}

		// 注册
		err = v1.TeamInfoRegister(req.TeamInfoBody, value.ProjectId, value.ProjectCode)
		if err != nil {
			return err
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}
	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}


//
// @Summary 更新班组信息
// @Description 更新班组信息
// @Tags user
// @Accept  json
// @Produce  json
// @Param body body v1.TeamInfoBody true "班组信息"
// @Success 200 {object} v1.TeamUpdateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/team [put]
func (c *Controller) TeamUpdate(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.TeamUpdateRequest{}
	resp := param_v1.TeamUpdateResponse{Result: base.Result{
		Code:    0,
		Message: "成功",
	}}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.TeamInfoBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		bytes, _ := json.Marshal(req)
		fmt.Printf("TeamUpdate read:%s\n", bytes)
		if req.TeamSysNo == 0 {
			return status.Error(10003, "参数错误 班组号不能为空")
		}
		if req.TeamSysNo == -100 {
			req.TeamName = "管理班组"
		}
		if req.TeamName == "" {
			return status.Error(10003, "参数错误 班组名称不能为空")
		}
		if req.TeamLeaderPhone == ""{
			return status.Error(10003, "参数错误 班组长联系电话不能为空")
		}
		if req.TeamLeaderName == "" {
			return status.Error(10003, "参数错误 班组长名称不能为空")
		}
		if req.CorpCode == "" && req.TeamSysNo != 100{
			return status.Error(10003, "参数错误 班组所属企业社会信用码不能为空")
		}
		if req.CorpName == "" && req.TeamSysNo != 100{
			return status.Error(10003, "参数错误 班组所属企业不能为空")
		}
		if req.Sn == "" {
			return status.Error(10003, "参数错误 设备sn码不能为空")
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// todo 设备类型
		value, err := utils.GetDeviceInfo(req.Sn)
		if err != nil {
			return err
		}

		// 注册
		err = v1.TeamUpdate(req.TeamInfoBody, value.ProjectId, value.ProjectCode)
		if err != nil {
			return err
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}
	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 删除班组信息
// @Description 删除班组信息
// @Tags user
// @Accept  json
// @Produce  json
// @Param body body v1.TeamDeleteBody true " "
// @Success 200 {object} v1.TeamDeleteResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/team/del [put]
func (c *Controller) TeamDelete(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.TeamDeleteRequest{}
	resp := param_v1.TeamDeleteResponse{Result: base.Result{
		Code:    0,
		Message: "成功",
	}}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.TeamDeleteBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		bytes, _ := json.Marshal(req)
		fmt.Printf("TeamDelete read:%s\n", bytes)
		if len(req.TeamSysNos) == 0 {
			return status.Error(10003, "参数错误 班组号不能为空")
		}
		if req.Sn == "" {
			return status.Error(10003, "参数错误 设备sn码不能为空")
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// todo 设备类型
		value, err := utils.GetDeviceInfo(req.Sn)
		if err != nil {
			return err
		}

		// 注册
		err = v1.TeamDelete(req.TeamSysNos, value.ProjectId, value.ProjectCode)
		if err != nil {
			return err
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}
	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

